@@ -3,19 +3,12 @@ use warnings;
 use strict;
 use Lingua::JA::Moji ':all';
 use Text::Fuzzy;
-use Time::HiRes 'time';
 use utf8;
 binmode STDOUT, ":utf8";
-my $max = 100;
-my $count;
 my $infile = '/home/ben/data/edrdg/edict';
 open my $in, "<:encoding(EUC-JP)", $infile or die $!;
 my @kana;
 while (<$in>) {
-    # $count++;
-    # if ($count > $max) {
-    # 	last;
-    # }
     my $kana;
     if (/\[(\p{InKana}+)\]/) {
 	$kana = $1;
@@ -23,9 +16,6 @@ while (<$in>) {
     elsif (/^(\p{InKana}+)/) {
 	$kana = $1;
     }
-    else {
-#	print "$infile:$.: no kana in $_.\n";
-    }
     if ($kana) {
 	$kana = kana2katakana ($kana);
 	push @kana, $kana;
@@ -42,10 +32,7 @@ exit;
 sub search
 {
     my ($silly) = @_;
-    my $start = time ();
     my $search = Text::Fuzzy->new ($silly);
-    $search->no_alphabet (0);
-#    $search->set_max_distance (3);
     my $n = $search->nearest (\@kana);
     if ($n >= 0) {
 	printf "$silly nearest is $kana[$n] (distance %d)\n",
@@ -55,7 +42,5 @@ sub search
 	printf "Nothing like '$silly' was found within the edit distance %d.\n",
 	    $search->get_max_distance ();
     }
-    my $end = time ();
-    printf "Fuzzy search took %g seconds.\n", ($end - $start);
 }
 
